@@ -3,33 +3,44 @@
 此模块定义机器人类，包含处于 model 层之下的 `SimpleMirai` 和建立在 model 层上的 `Mirai`。
 """
 import asyncio
+import contextlib
 import logging
 import sys
-import contextlib
-from typing import Callable, List, Type, Union
+from typing import (
+    Any, Awaitable, Callable, Dict, Iterable, List, Optional, Type, Union, cast
+)
 
-from mirai.adapters.base import Adapter, ApiProvider, AdapterInterface
+from mirai.adapters.base import Adapter, AdapterInterface, ApiProvider
 from mirai.asgi import ASGI, asgi_serve
-from mirai.bus import EventBus
+from mirai.bus import AbstractEventBus, EventBus
 from mirai.models.api import ApiModel
 from mirai.models.bus import ModelEventBus
-from mirai.models.events import Event
-from mirai.utils import Singleton, async_, async_call_with_exception
+from mirai.models.entities import (
+    Entity, Friend, Group, GroupMember, Permission, Subject
+)
+from mirai.models.events import Event, MessageEvent, TempMessage
+from mirai.models.message import MessageChain, MessageComponent
+from mirai.utils import Singleton
+
+__all__ = [
+    'Mirai', 'SimpleMirai', 'MiraiRunner', 'LifeSpan', 'Startup', 'Shutdown'
+]
 
 
-class SimpleMirai(ApiProvider, AdapterInterface):
+class SimpleMirai(ApiProvider, AdapterInterface, AbstractEventBus):
     """
     基于 adapter 和 bus，处于 model 层之下的机器人类。
 
     使用了 `__getattr__` 魔术方法，可以直接在对象上调用 API。
 
-    通过 `SimpleMirai` 调用 API 时，需注意此类不含 model 层封装，因此 API 名称与参数名称需与 mirai-api-http 中的定义相同，
+    通过 `SimpleMirai` 调用 API 时，需注意此类不含 model 层封装，
+    因此 API 名称与参数名称需与 mirai-api-http 中的定义相同，
     参数需要全部以具名参数的形式给出，并且需要指明使用的方法（GET/POST）。
 
     例如：
     ```py
     await bot.sendFriendMessage(target=12345678, messageChain=[
-        {"type":"Plain", "text":"Hello World!"}
+        {"type": "Plain", "text": "Hello World!"}
     ], method="POST")
     ```
 
@@ -37,12 +48,16 @@ class SimpleMirai(ApiProvider, AdapterInterface):
 
     对于名称的路由含有二级目录的 API，由于名称中含有斜杠，必须使用 `call_api` 调用，例如：
     ```py
-    file_list = await bot.call_api("file/list", id="", target=12345678, method="GET")
+    file_list = await bot.call_api(
+        "file/list", id="", target=12345678, method="GET"
+    )
     ```
     """
+    qq: int
+
     def __init__(self, qq: int, adapter: Adapter):
         """
-        `qq: int` QQ 号。
+        `qq: int` QQ 号。启用 Single Mode 时，可以随便传入，登陆后会自动获取正确的 QQ 号。
 
         `adapter: Adapter` 适配器，负责与 mirai-api-http 沟通，详见模块`mirai.adapters`。
 
@@ -54,25 +69,34 @@ class SimpleMirai(ApiProvider, AdapterInterface):
         self._bus = EventBus()
         self._adapter.register_event_bus(self._bus)
 
-        self.setup_functions = []
+    @property
+    def bus(self) -> EventBus:
+        return self._bus
 
-        self.logger = logging.getLogger(__name__)
+    def subscribe(self, event, func: Callable, priority: int = 0) -> None:
+        self._bus.subscribe(event, func, priority)
 
-    async def call_api(self, api: str, **params):
+    def unsubscribe(self, event, func: Callable) -> None:
+        self._bus.unsubscribe(event, func)
+
+    async def emit(self, event, *args, **kwargs) -> List[Awaitable[Any]]:
+        return await self._bus.emit(event, *args, **kwargs)
+
+    async def call_api(self, api: str, *args, **kwargs):
         """调用 API。
 
         `api: str` API 名称。
 
-        `**params` 参数。
+        `*args, **kwargs` 参数。
         """
-        return await async_(self._adapter.call_api(api, **params))
+        return await self._adapter.call_api(api, *args, **kwargs)
 
     def on(self, event: str, priority: int = 0) -> Callable:
         """注册事件处理器。
 
         `event: str` 事件名。
 
-        `priority: int = 0` 优先级，较小者优先。
+        `priority: int = 0` 优先级，小者优先。
 
         用法举例：
         ```py
@@ -81,15 +105,10 @@ class SimpleMirai(ApiProvider, AdapterInterface):
             print(f"收到来自{event['sender']['nickname']}的消息。")
         ```
         """
-        return self._bus.on(event, priority=priority)
-
-    def setup(self, func: Callable) -> Callable:
-        """注册机器人初始化处理器。"""
-        self.setup_functions.append(func)
-        return func
+        return self._bus.on(event)
 
     @property
-    def adapter_info(self):
+    def adapter_info(self) -> Dict[str, Any]:
         return self._adapter.adapter_info
 
     @contextlib.asynccontextmanager
@@ -109,15 +128,17 @@ class SimpleMirai(ApiProvider, AdapterInterface):
         self._adapter = adapter
         yield
         self._adapter = origin_adapter
-        # await adapter.logout()
+        await adapter.logout(False)
 
     async def startup(self):
         """开始运行机器人（立即返回）。"""
         await self._adapter.login(self.qq)
 
-        for func in self.setup_functions:
-            await async_call_with_exception(func)
+        if self._adapter.single_mode:
+            # Single Mode 下，QQ 号可以随便传入。这里从 session info 中获取正确的 QQ 号。
+            self.qq = (await self.call_api('sessionInfo'))['data']['qq']['id']
 
+        asyncio.create_task(self._adapter.emit("Startup", {'type': 'Startup'}))
         self._adapter.start()
 
     async def background(self):
@@ -126,11 +147,19 @@ class SimpleMirai(ApiProvider, AdapterInterface):
 
     async def shutdown(self):
         """结束运行机器人。"""
+        await asyncio.create_task(
+            self._adapter.emit("Shutdown", {'type': 'Shutdown'})
+        )
         await self._adapter.logout()
         self._adapter.shutdown()
 
     @property
-    def asgi(self):
+    def session(self) -> str:
+        """获取 session key，可用于调试。"""
+        return self._adapter.session
+
+    @property
+    def asgi(self) -> 'MiraiRunner':
         return MiraiRunner(self)
 
     def run(
@@ -166,8 +195,7 @@ class MiraiRunner(Singleton):
     runner.run(host='127.0.0.1', port=8000)
     ```
     """
-    _created = None
-    bots: List[SimpleMirai]
+    bots: Iterable[SimpleMirai]
     """运行的 SimpleMirai 对象。"""
     def __init__(self, *bots: SimpleMirai):
         """
@@ -258,12 +286,17 @@ class Mirai(SimpleMirai):
         super().__init__(qq=qq, adapter=adapter)
         # 将 bus 更换为 ModelEventBus
         adapter.unregister_event_bus(self._bus)
-        self._bus = ModelEventBus()
+        self._bus: ModelEventBus = ModelEventBus()
         adapter.register_event_bus(self._bus.base_bus)
 
+    @property
+    def bus(self) -> ModelEventBus:
+        return self._bus
+
     def on(
         self,
         event_type: Union[Type[Event], str],
+        priority: int = 0,
     ) -> Callable:
         """注册事件处理器。
 
@@ -278,7 +311,7 @@ class Mirai(SimpleMirai):
             print(f"收到来自{event.sender.nickname}的消息。")
         ```
         """
-        return self._bus.on(event_type=event_type)
+        return self._bus.on(event_type, priority)
 
     def api(self, api: str) -> ApiModel.Proxy:
         """获取 API Proxy 对象。
@@ -294,3 +327,135 @@ class Mirai(SimpleMirai):
 
     def __getattr__(self, api: str) -> ApiModel.Proxy:
         return self.api(api)
+
+    async def send(
+        self,
+        target: Union[Entity, MessageEvent],
+        message: Union[MessageChain, List[Union[MessageComponent, str]], MessageComponent, str],
+        quote: bool = False
+    ) -> int:
+        """发送消息。可以从 `Friend` `Group` 等对象，或者从 `MessageEvent` 中自动识别消息发送对象。
+
+        `target: Union[Entity, MessageEvent]` 目标对象。
+
+        `message: Union[MessageChain, List[Union[MessageComponent, str]], str]` 发送的消息。
+
+        `quote: bool = False` 是否以回复消息的形式发送。
+
+        返回值：int 发送的消息的 message_id。
+        """
+        # 识别消息发送对象
+        if isinstance(target, TempMessage):
+            quoting = target.message_chain.message_id if quote else None
+            return (
+                await self.send_temp_message(
+                    qq=target.sender.id,
+                    group=target.group.id,
+                    message_chain=message,
+                    quote=quoting
+                )
+            ).message_id
+        else:
+            if isinstance(target, MessageEvent):
+                quoting = target.message_chain.message_id if quote else None
+                target = target.sender
+            else:
+                quoting = None
+
+            if isinstance(target, Friend):
+                send_message = self.send_friend_message
+                id_ = target.id
+            elif isinstance(target, Group):
+                send_message = self.send_group_message
+                id_ = target.id
+            elif isinstance(target, GroupMember):
+                send_message = self.send_group_message
+                id_ = target.group.id
+            else:
+                raise ValueError(f"{target} 不是有效的消息发送对象。")
+
+            return (
+                await
+                send_message(target=id_, message_chain=message, quote=quoting)
+            ).message_id
+
+    async def get_friend(self, id_: int) -> Optional[Friend]:
+        """获取好友对象。
+
+        `id: int` 好友 ID。
+        """
+        friend_list = await self.friend_list.get()
+        if not friend_list:
+            return None
+        for friend in cast(List[Friend], friend_list):
+            if friend.id == id_:
+                return friend
+        return None
+
+    async def get_group(self, id_: int) -> Optional[Group]:
+        """获取群组对象。
+
+        `id: int` 群组 ID。
+        """
+        group_list = await self.group_list.get()
+        if not group_list:
+            return None
+        for group in cast(List[Group], group_list):
+            if group.id == id_:
+                return group
+        return None
+
+    async def get_group_member(self, group: Union[Group, int],
+                               id_: int) -> Optional[GroupMember]:
+        """获取群成员对象。
+
+        `group: Union[Group, int]` 群组对象或群组 ID。
+
+        `id: int` 群成员 ID。
+        """
+        if isinstance(group, Group):
+            group = group.id
+        member_list = await self.member_list.get(group)
+        if not member_list:
+            return None
+        for member in cast(List[GroupMember], member_list):
+            if member.id == id_:
+                return member
+        return None
+
+    async def get_entity(self, subject: Subject) -> Optional[Entity]:
+        """获取实体对象。
+
+        `subject: Subject` 实体对象。
+        """
+        if subject.kind == 'Friend':
+            return await self.get_friend(subject.id)
+        elif subject.kind == 'Group':
+            return await self.get_group(subject.id)
+        else:
+            return None
+
+    async def is_admin(self, group: Group) -> bool:
+        """判断机器人在群组中是否是管理员。
+
+        `group: Group` 群组对象。
+        """
+        return group.permission in (Permission.Administrator, Permission.Owner)
+
+
+class LifeSpan(Event):
+    """生命周期事件。"""
+    type: str = 'LifeSpan'
+    """事件名。"""
+
+
+class Startup(LifeSpan):
+    """启动事件。"""
+    type: str = 'Startup'
+    """事件名。"""
+
+
+class Shutdown(LifeSpan):
+    """关闭事件。"""
+    type: str = 'Shutdown'
+    """事件名。"""
