@@ -4,13 +4,15 @@
 
 此处的事件总线不包含 model 层封装。包含 model 层封装的版本，请参见模块 `mirai.models.bus`。
 """
+import abc
 import asyncio
 import inspect
 import logging
 from collections import defaultdict
-from typing import Any, Awaitable, Callable, Iterable, List, Optional
+from typing import Any, Awaitable, Callable, Dict, Iterable, List, Optional
 
-from mirai.utils import async_call_with_exception
+from mirai.exceptions import SkipExecution, StopExecution, StopPropagation
+from mirai.utils import PriorityDict, async_with_exception
 
 logger = logging.getLogger(__name__)
 
@@ -23,8 +25,8 @@ def event_chain_separator(sep: str = '.'):
     def generator(event: str):
         while True:
             yield event
-            event, *sub_event = event.rsplit(sep, maxsplit=1) # 由下到上依次触发
-            if not sub_event: # 顶层事件触发完成
+            event, *sub_event = event.rsplit(sep, maxsplit=1)  # 由下到上依次触发
+            if not sub_event:  # 顶层事件触发完成
                 break
 
     return generator
@@ -35,7 +37,63 @@ def event_chain_single(event: str):
     yield event
 
 
-class EventBus(object):
+class AbstractEventBus:
+    """抽象事件总线。
+
+    事件总线的基类。
+    """
+    @abc.abstractmethod
+    def subscribe(self, event, func: Callable, priority: int = 0) -> None:
+        """注册事件处理器。
+
+        `event` 事件名。
+
+        `func: Callable` 事件处理器。
+
+        `priority: int = 0` 优先级，小者优先。
+        """
+
+    @abc.abstractmethod
+    def unsubscribe(self, event, func: Callable) -> None:
+        """移除事件处理器。
+
+        `event` 事件名。
+
+        `func: Callable` 事件处理器。
+        """
+
+    @abc.abstractmethod
+    def on(self, event, priority: int = 0) -> Callable:
+        """以装饰器的方式注册事件处理器。
+
+        `event` 事件名。
+
+        `priority: int = 0` 优先级，小者优先。
+
+        例如：
+        ```py
+        @bus.on('Event.MyEvent')
+        def my_event_handler(event):
+            print(event)
+        ```
+        """
+
+    @abc.abstractmethod
+    async def emit(self, event, *args, **kwargs) -> List[Awaitable[Any]]:
+        """触发一个事件。
+
+        异步执行说明：`await emit` 时执行事件处理器，所有事件处理器执行完后，并行运行所有快速响应。
+
+        `event` 要触发的事件名称。
+
+        `*args, **kwargs` 传递给事件处理器的参数。
+
+        Returns: `List[Awaitable[Any]]`
+            所有事件处理器的快速响应协程的 Task。
+        """
+
+
+class EventBus(AbstractEventBus):
     """事件总线。
 
     事件总线提供了一个简单的方法，用于分发事件。事件处理器可以通过 `subscribe` 或 `on` 注册事件，
@@ -58,17 +116,20 @@ class EventBus(object):
             一个函数，输入事件名，返回一个生成此事件所在事件链的全部事件的事件名的生成器，
             默认行为是事件链只包含单一事件。
         """
-        self._subscribers = defaultdict(set)
+        self._subscribers: Dict[
+            str, PriorityDict[Callable]] = defaultdict(PriorityDict)
         self.event_chain_generator = event_chain_generator
 
-    def subscribe(self, event: str, func: Callable) -> None:
+    def subscribe(self, event: str, func: Callable, priority: int = 0) -> None:
         """注册事件处理器。
 
         `event: str` 事件名。
 
         `func: Callable` 事件处理器。
+
+        `priority: int = 0` 优先级，小者优先。
         """
-        self._subscribers[event].add(func)
+        self._subscribers[event].add(priority, func)
 
     def unsubscribe(self, event: str, func: Callable) -> None:
         """移除事件处理器。
@@ -80,13 +141,15 @@ class EventBus(object):
         try:
             self._subscribers[event].remove(func)
         except KeyError:
-            logger.warn(f'试图移除事件 `{event}` 的一个不存在的事件处理器 `{func}`。')
+            logger.warning(f'试图移除事件 `{event}` 的一个不存在的事件处理器 `{func}`。')
 
-    def on(self, event: str) -> Callable:
+    def on(self, event: str, priority: int = 0) -> Callable:
         """以装饰器的方式注册事件处理器。
 
         `event: str` 事件名。
 
+        `priority: int = 0` 优先级，小者优先。
+
         例如：
         ```py
         @bus.on('Event.MyEvent')
@@ -95,7 +158,7 @@ class EventBus(object):
         ```
         """
         def decorator(func: Callable) -> Callable:
-            self.subscribe(event, func)
+            self.subscribe(event, func, priority)
             return func
 
         return decorator
@@ -113,20 +176,31 @@ class EventBus(object):
             所有事件处理器的快速响应协程的 Task。
         """
         async def call(f) -> Optional[Awaitable[Any]]:
-            result = await async_call_with_exception(f, *args, **kwargs)
+            result = await async_with_exception(f(*args, **kwargs))
             # 快速响应：如果事件处理器返回一个协程，那么立即运行这个协程。
             if inspect.isawaitable(result):
-                return result
-            else: # 当不使用快速响应时，返回值无意义。
+                return async_with_exception(result)
+            else:  # 当不使用快速响应时，返回值无意义。
                 return None
 
-        coros = []
-        for m_event in self.event_chain_generator(event):
-            coros += [(await call(f)) for f in self._subscribers[m_event]]
-        coros = filter(None, coros) # 只保留快速响应的返回值。
+        coros: List[Optional[Awaitable[Any]]] = []
+        try:
+            for m_event in self.event_chain_generator(event):
+                try:
+                    # 使用 list 避免 _subscribers 被改变引起错误。
+                    for listeners in list(self._subscribers[m_event]):
+                        try:
+                            callee = (call(f) for f in listeners)
+                            coros += await asyncio.gather(*callee)
+                        except SkipExecution:
+                            continue
+                except StopExecution:
+                    continue
+        except StopPropagation:
+            pass
 
-        results = list(coros)
-        return list(map(lambda coro: asyncio.create_task(coro), results))
+        # 只保留快速响应的返回值。
+        return [asyncio.create_task(coro) for coro in filter(None, coros)]
 
     @classmethod
     def get_default_bus(cls):
