@@ -23,6 +23,7 @@ API_ERROR_FMT = {
     20: 'Bot 被禁言。',
     30: '消息过长。',
     400: '参数错误。',
+    500: 'mirai 内部错误。',
 }
 
 
@@ -31,9 +32,25 @@ class ApiError(RuntimeError):
 
     `code: int` mirai-api-http 的 API 状态码。
     """
-    def __init__(self, code: int):
+    def __init__(self, response: dict):
+        code = response['code']
         self.code = code
-        self.args = (code, f'[ERROR {code}]' + API_ERROR_FMT[code])
+        self.args = (
+            code, f'[ERROR {code}]' + API_ERROR_FMT.get(code, ''),
+            response.get('msg', '')
+        )
+
+
+class StopPropagation(Exception):
+    """终止事件处理器执行，并停止事件向上传播。"""
+
+
+class StopExecution(Exception):
+    """终止事件处理器执行，但不阻止事件向上传播。"""
+
+
+class SkipExecution(Exception):
+    """跳过同优先度的事件处理器，进入下一优先度。"""
 
 
 def print_exception(e: Exception):
@@ -42,4 +59,4 @@ def print_exception(e: Exception):
     traceback.print_exception(type(e), e, e.__traceback__)
 
 
-__all__ = ['NetworkError', 'ApiError', 'LoginError', 'print_exception']
+__all__ = ['NetworkError', 'ApiError', 'print_exception']
