@@ -9,13 +9,16 @@ import re
 from datetime import datetime
 from json import loads as json_loads
 from pathlib import Path
-from typing import List, Optional, Union
+from typing import List, Optional, Union, cast
 
 import aiofiles
 import httpx
-from mirai.models.base import MiraiBaseModel, MiraiIndexedMetaclass, MiraiIndexedModel
+from pydantic import HttpUrl, validator
+
+from mirai.models.base import (
+    MiraiBaseModel, MiraiIndexedMetaclass, MiraiIndexedModel
+)
 from mirai.utils import KMP
-from pydantic import Field, HttpUrl, validator
 
 logger = logging.getLogger(__name__)
 
@@ -25,7 +28,8 @@ def serialize(s: str) -> str:
 
     `s: str` 待转义的字符串。
     """
-    return re.sub(r'[\[\]:,\\\n\r]', lambda match: '\\' + match.group(0), s)
+    return re.sub(r'[\[\]:,\\]', lambda match: '\\' + match.group(0),
+                  s).replace('\n', '\\n').replace('\r', '\\r')
 
 
 def deserialize(s: str) -> str:
@@ -33,7 +37,10 @@ def deserialize(s: str) -> str:
 
     `s: str` 待去转义的字符串。
     """
-    return re.sub(r'\\([\[\]:,\\nr])', lambda match: match.group(1), s)
+    return re.sub(
+        r'\\([\[\]:,\\])', lambda match: match.group(1),
+        s.replace('\\n', '\n').replace('\\r', '\r')
+    )
 
 
 class MessageComponentMetaclass(MiraiIndexedMetaclass):
@@ -56,6 +63,8 @@ class MessageComponentMetaclass(MiraiIndexedMetaclass):
                     new_cls.__parameter_names__ = list(
                         new_cls.__annotations__
                     )[1:]
+                else:
+                    new_cls.__parameter_names__ = []
                 break
 
         return new_cls
@@ -68,6 +77,14 @@ class MessageComponent(MiraiIndexedModel, metaclass=MessageComponentMetaclass):
     def __str__(self):
         return ''
 
+    def __repr__(self):
+        return self.__class__.__name__ + '(' + ', '.join(
+            (
+                f'{k}={repr(v)}'
+                for k, v in self.__dict__.items() if k != 'type' and v
+            )
+        ) + ')'
+
     def __init__(self, *args, **kwargs):
         # 解析参数列表，将位置参数转化为具名参数
         parameter_names = self.__parameter_names__
@@ -76,7 +93,7 @@ class MessageComponent(MiraiIndexedModel, metaclass=MessageComponentMetaclass):
                 f'`{self.type}`需要{len(parameter_names)}个参数，但传入了{len(args)}个。'
             )
         for name, value in zip(parameter_names, args):
-            if kwargs.get(name):
+            if name in kwargs:
                 raise TypeError(f'在 `{self.type}` 中，具名参数 `{name}` 与位置参数重复。')
             else:
                 kwargs[name] = value
@@ -202,6 +219,9 @@ class MessageChain(MiraiBaseModel):
 
     @validator('__root__', always=True, pre=True)
     def _parse_component(cls, msg_chain):
+        if isinstance(msg_chain,
+                      str) or isinstance(msg_chain, MessageComponent):
+            msg_chain = [msg_chain]
         if not msg_chain:
             msg_chain = []
         return cls._parse_message_chain(msg_chain)
@@ -219,7 +239,10 @@ class MessageChain(MiraiBaseModel):
         super().__init__(__root__=__root__)
 
     def __str__(self):
-        return "".join((str(component) for component in self.__root__))
+        return "".join(str(component) for component in self.__root__)
+
+    def __repr__(self):
+        return f'{self.__class__.__name__}({self.__root__!r})'
 
     def __iter__(self):
         yield from self.__root__
@@ -232,7 +255,8 @@ class MessageChain(MiraiBaseModel):
         # 索引对象为 MessageComponent 类，返回所有对应 component
         elif isinstance(index, type):
             return [
-                component for component in self if type(component) == index
+                cast(MessageComponent, component) for component in self
+                if type(component) == index
             ]
         # 索引对象为 MessageComponent 和 int 构成的 slice， 返回指定数量的 component
         elif isinstance(index, slice):
@@ -244,24 +268,26 @@ class MessageChain(MiraiBaseModel):
                 component
                 for component, _ in zip(components, range(index.stop))
             ]
+        raise TypeError(f"消息链索引需为 int 或 MessageComponent，当前类型：{type(index)}")
 
     def __contains__(self, sub) -> bool:
-        if isinstance(sub, type): # 检测消息链中是否有某种类型的对象
+        if isinstance(sub, type):  # 检测消息链中是否有某种类型的对象
             for i in self:
                 if type(i) == sub:
                     return True
             else:
                 return False
-        elif isinstance(sub, MessageComponent): # 检查消息链中是否有某个组件
+        elif isinstance(sub, MessageComponent):  # 检查消息链中是否有某个组件
             for i in self:
                 if i == sub:
                     return True
             else:
                 return False
-        elif isinstance(sub, MessageChain): # 检查消息链中是否有某个子消息链
+        elif isinstance(sub, MessageChain):  # 检查消息链中是否有某个子消息链
             return bool(KMP(self, sub))
-        elif isinstance(sub, str): # 检查消息中有无指定字符串子串
+        elif isinstance(sub, str):  # 检查消息中有无指定字符串子串
             return sub in deserialize(str(self))
+        raise TypeError(f"类型不匹配，当前类型：{type(sub)}")
 
     def __ge__(self, other):
         return other in self
@@ -273,13 +299,13 @@ class MessageChain(MiraiBaseModel):
     def source(self) -> 'Source':
         """获取消息链中的 `Source` 对象。"""
         source = self[Source:1]
-        return source[0] if source else None
+        return cast(list, source)[0] if source else None
 
     @property
     def message_id(self) -> int:
         """获取消息链的 message_id。"""
         source = self.source
-        return source.id if source else None
+        return source.id if source else -1
 
 
 class Source(MessageComponent):
@@ -301,23 +327,25 @@ class Plain(MessageComponent):
     def __str__(self):
         return serialize(self.text)
 
+    def __repr__(self):
+        return f'Plain({self.text!r})'
+
 
 class Quote(MessageComponent):
     """引用。"""
     type: str = "Quote"
     """消息组件类型。"""
-    id: Optional[int]
+    id: Optional[int] = None
     """被引用回复的原消息的 message_id。"""
-    group_id: Optional[int] = Field(alias='groupId')
+    group_id: Optional[int] = None
     """被引用回复的原消息所接收的群号，当为好友消息时为0。"""
-    sender_id: Optional[int] = Field(alias='senderId')
+    sender_id: Optional[int] = None
     """被引用回复的原消息的发送者的QQ号。"""
-    target_id: Optional[int] = Field(alias='targetId')
+    target_id: Optional[int] = None
     """被引用回复的原消息的接收者者的QQ号（或群号）。"""
     origin: MessageChain
     """被引用回复的原消息的消息链对象。"""
     @validator("origin", always=True, pre=True)
-    @classmethod
     def origin_formater(cls, v):
         return MessageChain.parse_obj(v)
 
@@ -349,9 +377,9 @@ class Face(MessageComponent):
     """表情。"""
     type: str = "Face"
     """消息组件类型。"""
-    face_id: Optional[int]
+    face_id: Optional[int] = None
     """QQ 表情编号，可选，优先度高于 name。"""
-    name: Optional[str]
+    name: Optional[str] = None
     """QQ表情拼音，可选。"""
     def __eq__(self, other):
         return isinstance(other, Face) and self.face_id == other.face_id
@@ -364,12 +392,12 @@ class Image(MessageComponent):
     """图片。"""
     type: str = "Image"
     """消息组件类型。"""
-    image_id: Optional[str]
+    image_id: Optional[str] = None
     """图片的 image_id，群图片与好友图片格式不同。不为空时将忽略 url 属性。"""
     url: Optional[HttpUrl] = None
     """图片的 URL，发送时可作网络图片的链接；接收时为腾讯图片服务器的链接，可用于图片下载。"""
-    path: Optional[str] = None
-    """图片的路径，发送本地图片，路径相对于 mirai-console 的目录，也可传入绝对路径。"""
+    path: Optional[Union[str, Path]] = None
+    """图片的路径，发送本地图片。"""
     base64: Optional[str] = None
     """图片的 Base64 编码。"""
     def __eq__(self, other):
@@ -380,12 +408,23 @@ class Image(MessageComponent):
     def __str__(self):
         return f"[mirai:image:{self.image_id}]"
 
+    @validator('path')
+    def validate_path(cls, path: Optional[Union[str, Path]]):
+        """修复 path 参数的行为，使之相对于 YiriMirai 的启动路径。"""
+        if path:
+            try:
+                return str(Path(path).resolve(strict=True))
+            except FileNotFoundError:
+                raise ValueError(f"无效路径：{path}")
+        else:
+            return path
+
     @property
     def uuid(self):
         image_id = self.image_id
-        if image_id[0] == '{': # 群图片
+        if image_id[0] == '{':  # 群图片
             image_id = image_id[1:37]
-        elif image_id[0] == '/': # 好友图片
+        elif image_id[0] == '/':  # 好友图片
             image_id = image_id[1:]
         return image_id
 
@@ -417,6 +456,10 @@ class Image(MessageComponent):
 
         `determine_type: bool = True` 是否自动根据图片类型确定拓展名。
         """
+        if not self.url:
+            logger.warning(f'图片 `{self.uuid}` 无 url 参数，下载失败。')
+            return
+
         async with httpx.AsyncClient() as client:
             response = await client.get(self.url)
             response.raise_for_status()
@@ -425,7 +468,9 @@ class Image(MessageComponent):
             if filename:
                 path = Path(filename)
                 if determine_type:
-                    path = path.with_suffix('.' + imghdr.what(None, content))
+                    path = path.with_suffix(
+                        '.' + str(imghdr.what(None, content))
+                    )
                 path.parent.mkdir(parents=True, exist_ok=True)
             elif directory:
                 path = Path(directory)
@@ -444,7 +489,7 @@ class Image(MessageComponent):
         cls,
         filename: Optional[Union[str, Path]] = None,
         content: Optional[bytes] = None,
-    ) -> "Image":
+    ):
         """从本地文件路径加载图片，以 base64 的形式传递。
 
         `filename: Optional[Union[str, Path]] = None` 从本地文件路径加载图片，与 `content` 二选一。
@@ -462,6 +507,14 @@ class Image(MessageComponent):
         img = cls(base64=base64.b64encode(content).decode())
         return img
 
+    @classmethod
+    def from_unsafe_path(cls, path: Union[str, Path]):
+        """从不安全的路径加载图片。
+
+        `path: Union[str, Path]` 从不安全的路径加载图片。
+        """
+        return cls.construct(path=str(path))
+
 
 class Xml(MessageComponent):
     """XML。"""
@@ -560,7 +613,7 @@ class FlashImage(Image):
     """闪照。"""
     type: str = "FlashImage"
     """消息组件类型。"""
-    image_id: Optional[str]
+    image_id: Optional[str] = None
     """图片的 image_id，群图片与好友图片格式不同。不为空时将忽略 url 属性。"""
     url: Optional[HttpUrl] = None
     """图片的 URL，发送时可作网络图片的链接；接收时为腾讯图片服务器的链接，可用于图片下载。"""
@@ -584,9 +637,20 @@ class Voice(MessageComponent):
     url: Optional[str] = None
     """语音的 URL，发送时可作网络语音的链接；接收时为腾讯语音服务器的链接，可用于语音下载。"""
     path: Optional[str] = None
-    """语音的路径，发送本地语音，路径相对于 `plugins/MiraiAPIHTTP/voices`。"""
+    """语音的路径，发送本地语音。"""
     base64: Optional[str] = None
     """语音的 Base64 编码。"""
+    @validator('path')
+    def validate_path(cls, path: Optional[str]):
+        """修复 path 参数的行为，使之相对于 YiriMirai 的启动路径。"""
+        if path:
+            try:
+                return str(Path(path).resolve(strict=True))
+            except FileNotFoundError:
+                raise ValueError(f"无效路径：{path}")
+        else:
+            return path
+
     async def download(self, filename=None, directory=None):
         """下载语音到本地。
 
@@ -596,6 +660,10 @@ class Voice(MessageComponent):
 
         `directory = None` 下载到本地的文件夹路径。与 `filename` 二选一。
         """
+        if not self.url:
+            logger.warning(f'语音 `{self.voice_id}` 无 url 参数，下载失败。')
+            return
+
         async with httpx.AsyncClient() as client:
             response = await client.get(self.url)
             response.raise_for_status()
@@ -619,7 +687,7 @@ class Voice(MessageComponent):
         cls,
         filename: Optional[Union[str, Path]] = None,
         content: Optional[bytes] = None,
-    ) -> "Image":
+    ) -> "Voice":
         """从本地文件路径加载语音，以 base64 的形式传递。
 
         `filename: Optional[Union[str, Path]] = None` 从本地文件路径加载图片，与 `content` 二选一。
@@ -702,6 +770,19 @@ class File(MessageComponent):
     """文件大小。"""
 
 
+class MiraiCode(MessageComponent):
+    """Mirai 码。"""
+    type: str = "MiraiCode"
+    """消息组件类型。"""
+    code: str
+    """Mirai 码。"""
+    def __str__(self):
+        return serialize(self.code)
+
+    def __repr__(self):
+        return f'MiraiCode({self.code!r})'
+
+
 __all__ = [
     'App',
     'At',
